@@ -52,10 +52,6 @@ ui = fluidPage(
             actionButton(
               'reset_to_def',
               label = 'Reset ALL to defaults'),
-            checkboxInput(
-              'normalizeInput',
-              'Normalize for peak amplitude',
-              value = TRUE),
             numericInput(
               'windowLength',
               'Window length, ms ("windowLength")',
@@ -477,6 +473,13 @@ ui = fluidPage(
                           "Heat" = "heat.colors",
                           "Black & white" = "bw"),
               selected = 'bw', inline = TRUE, width = NULL),
+            numericInput(
+              'nColors',
+              'Number of colors in the palette',
+              value = defaults_analyze['nColors', 'default'],
+              min = defaults_analyze['nColors', 'low'],
+              max = defaults_analyze['nColors', 'high'],
+              step = defaults_analyze['nColors', 'step']),
             sliderInput(
               'specContrast',
               'Contrast ("contrast")',
@@ -490,7 +493,52 @@ ui = fluidPage(
               value = defaults_analyze['specBrightness', 'default'],
               min = defaults_analyze['specBrightness', 'low'],
               max = defaults_analyze['specBrightness', 'high'],
-              step = defaults_analyze['specBrightness', 'step'])
+              step = defaults_analyze['specBrightness', 'step']),
+            sliderInput(
+              'blur_freq',
+              'Blur: frequency (Hz)',
+              value = defaults_analyze['blur_freq', 'default'],
+              min = defaults_analyze['blur_freq', 'low'],
+              max = defaults_analyze['blur_freq', 'high'],
+              step = defaults_analyze['blur_freq', 'step']),
+            sliderInput(
+              'blur_time',
+              'Blur: time (ms)',
+              value = defaults_analyze['blur_time', 'default'],
+              min = defaults_analyze['blur_time', 'low'],
+              max = defaults_analyze['blur_time', 'high'],
+              step = defaults_analyze['blur_time', 'step']),
+
+            shinyBS::bsCollapsePanel(
+              "Advanced",
+              radioButtons(
+                inputId = 'specType',
+                label = 'Spectrogram method',
+                choices = c("Ordinary FFT" = "spectrum",
+                            "Reassigned" = "reassigned"),
+                selected = 'spectrum', inline = TRUE, width = NULL),
+              sliderInput(
+                'reass_cex',
+                'Point size (reassigned spectrogram only)',
+                value = defaults_analyze['reass_cex', 'default'],
+                min = defaults_analyze['reass_cex', 'low'],
+                max = defaults_analyze['reass_cex', 'high'],
+                step = defaults_analyze['reass_cex', 'step']),
+              numericInput(
+                'reass_windowLength',
+                'Window length, ms (reassigned only)',
+                value = defaults_analyze['reass_windowLength', 'default'],
+                min = defaults_analyze['reass_windowLength', 'low'],
+                max = defaults_analyze['reass_windowLength', 'high'],
+                step = defaults_analyze['reass_windowLength', 'step']),
+              numericInput(
+                'reass_step',
+                'Step, ms (reassigned only)',
+                value = defaults_analyze['reass_step', 'default'],
+                min = defaults_analyze['reass_step', 'low'],
+                max = defaults_analyze['reass_step', 'high'],
+                step = defaults_analyze['reass_step', 'step'])
+            )
           ),
 
           tabPanel(
@@ -500,13 +548,6 @@ ui = fluidPage(
               'Oscillogram type ("osc")',
               choices = c('linear', 'dB'),
               selected = 'linear', multiple = FALSE),
-            sliderInput(
-              'osc_height',
-              'Oscillogram height, px ("heights")',
-              value = defaults_analyze['osc_height', 'default'],
-              min = defaults_analyze['osc_height', 'low'],
-              max = defaults_analyze['osc_height', 'high'],
-              step = defaults_analyze['osc_height', 'step']),
             sliderInput(
               'osc_maxPoints',
               'Max number of pixels, 10^',
@@ -674,7 +715,15 @@ ui = fluidPage(
             )
           ),
 
-          plotOutput('oscillogram', height = '100px')  # default size
+          tags$div(
+            id = 'oscDiv',
+            plotOutput('oscillogram'),
+            plotOutput(
+              'oscOver',
+              click = "osc_click"
+            )
+          ),
+          # plotOutput('oscillogram', height = '100px')  # default size
         )
       )
 
